@@ -31,7 +31,7 @@
 #pod
 #pod =for :that If you are using a C<TextTemplater>-based plugin, read the
 #pod L<manual|Dist::Zilla::Role::TextTemplater::Manual>. General topics like getting source, building, installing, bug
-#pod reporting and some others are covered in the L<readme|Dist::Zilla::Role::TextTemplater::ReadMe>.
+#pod reporting and some others are covered in the F<README>.
 #pod
 #pod =head1 SYNOPSIS
 #pod
@@ -71,13 +71,14 @@ use Moose::Role;
 use namespace::autoclean;
 
 # ABSTRACT: Have text templating capabilities in your Dist::Zilla plugin
-our $VERSION = '0.005'; # VERSION
+our $VERSION = '0.006'; # VERSION
 
 with 'Dist::Zilla::Role::ErrorLogger' => { -version => 0.005 };
 
-use Text::Template qw{};
 use Carp qw{ croak };
+use Dist::Zilla::File::OnDisk;
 use List::Util qw{ min max };
+use Text::Template qw{};
 
 # --------------------------------------------------------------------------------------------------
 
@@ -184,26 +185,6 @@ has broken => (
     init_arg    => undef,
 );
 
-#   Claculate range of lines to show. $line is a lineno of focus line, $ctx is number of context
-#   lines to show above and below the focus line, $min and $max defined full range. It is usually
-#   1 and number of lines in the text. Note: linunumbers are one-based.
-sub __range($$$$) {
-    my ( $line, $ctx, $min, $max ) = @_;
-    if ( $line > 0 ) {
-        my $first = max( $line - $ctx, $min );  # First line to show.
-        my $last  = min( $line + $ctx, $max );  # Last line to show.
-        if ( $first - $min < $ctx ) {   # If number of skipped lines at top less than context,
-            $first = $min;              # do not skip lines.
-        };
-        if ( $max - $last < $ctx ) {    # If number of skipped lines at botton less than context,
-            $last = $max;               #  do not skip lines.
-        };
-        return ( $first, $last );
-    } else {
-        return ( 0, -1 );
-    };
-};
-
 sub _build_broken {
     my ( $self ) = @_;
     return sub {
@@ -215,36 +196,15 @@ sub _build_broken {
         if ( $error =~ m{ at \Q$tfile\E line (\d+)} ) {     # if the problem occurs in template.
             $eline = $1;
         };
-        my @template = split( "\n", $self->tt_template );   # Template text.
         chomp( $error );
         $self->log_error( $error );
         $self->log_error( "Problem code fragment begins at $self->{ tt_filename } line $tline." );
         $self->log_error( "$tfile:" );
-        #   TODO: Chop too long lines.
-        my $ctx  = 2;                       # Context (number of lines to show above and below).
-        my $w    = length( 0 + @template ); # Width of linenumber column.
-        my $text = '    %0' . $w . 'd: %s'; # Template text line format.
-        my $note = '    ' . ( ' ' x $w ) . '  ^^^ %s ^^^';              # Notice line format.
-        my $skip = '    ' . ( ' ' x $w ) . '  ...skipped %d lines...';  # "Skipped" notice format.
-        my ( $t0, $t1 ) = __range( $tline, $ctx,       1, 0 + @template );
-        my ( $e0, $e1 ) = __range( $eline, $ctx, $t1 + 1, 0 + @template );
-        my $last = 0;       # Number of last shown line.
-        for my $line ( $t0 .. $t1, $e0 .. $e1 ) {
-            if ( $line != $last + 1 ) {
-                $self->log_error( [ $skip, $line - $last - 1 ] );
-            };
-            $self->log_error( [ $text, $line, $template[ $line - 1 ] ] );
-            if ( $line == $tline ) {
-                $self->log_error( [ $note, 'Code fragment begins in line above' ] );
-            };
-            if ( $line == $eline ) {
-                $self->log_error( [ $note, 'Code fragment died in line above' ] );
-            };
-            $last = $line;
-        };
-        if ( $last < @template ) {
-            $self->log_error( [ $skip, @template - $last ] );
-        };
+        $self->_show_context(
+            $self->tt_template,
+            $tline => 'Code fragment begins in line above',
+            $eline => 'Code fragment died in line above',
+        );
         # TODO: Do we need $self->abort() here?
     };
 };
@@ -286,6 +246,25 @@ has tt_template => (
 
 # --------------------------------------------------------------------------------------------------
 
+#pod =attr tt_context
+#pod
+#pod Number of template lines to show above and below error line in error messages.
+#pod
+#pod C<Int>, read-only, not an init arg, default value 2.
+#pod
+#pod There is no (official) way to change the attribute value now. Let me know if you need it.
+#pod
+#pod =cut
+
+has tt_context => (
+    is          => 'ro',
+    isa         => 'Int',
+    init_arg    => undef,
+    default     => 2,
+);
+
+# --------------------------------------------------------------------------------------------------
+
 #pod =method mvp_multivalue_args
 #pod
 #pod The method tells C<Dist::Zilla> that C<prepend> is a multi-value option.
@@ -370,6 +349,32 @@ sub fill_in_string {
                 $self->log( $msg );
             };
             $result = $tt->fill_in( %args );
+            if ( not defined( $result ) ) {
+                #   There are few error message which can be generated by "Text::Template":
+                #       Unmatched close brace at line $lineno
+                #       End of data inside program text that began at line $prog_start
+                #       Couldn't open file $fn: $!
+                #   The latter cannot occur because we always fill in strings, never files.
+                #   The problem is that these error messages does not include template name. Let us
+                #   try to make them more user-friendly.
+                my $open  = qr{\QEnd of data inside program text that began\E}x;
+                my $close = qr{\QUnmatched close brace\E}x;
+                my $known = qr{\A (?: ($open) | $close ) \s at \s line \s (\d+) \z }x;
+                if ( $Text::Template::ERROR =~ $known ) {
+                    my ( $type, $line ) = ( $1, $2 );
+                    my $msg = $type ? 'Unmatched opening delimiter' : 'Unmatched closing delimiter';
+                        # ^ `Text::Template` error message "End of data inside program text that
+                        #   began at…" is too long at too complicated. Let us replace it with
+                        #   simpler one.
+                    $self->log_error( [ '%s at %s line %d.', $msg, $filename, $line ] );
+                    $self->log_error( "$filename:" );
+                    $self->_show_context(
+                        $self->tt_template,
+                        $line => "$msg in line above",
+                    );
+                    $result = '';   # Define result to avoid double error reporting.
+                };
+            };
         };
         $error = $Text::Template::ERROR;
     }
@@ -384,23 +389,147 @@ sub fill_in_string {
 
 #pod =method fill_in_file
 #pod
-#pod     $file = Dist::Zilla::File::OnDisk->new( { … } );
+#pod     $file = Dist::Zilla::File::OnDisk->new( { … } );    # or
+#pod     $file = Dist::Zilla::File::InMemory->new( { … } );  # or
+#pod     $file = Dist::Zilla::File::FromCode->new( { … } );  # or
+#pod     $file = Path::Tiny->new( 'filename' );              # or
+#pod     $file = Path::Class::File->new( 'filename' );       # or
+#pod     $file = 'filename.ext';
+#pod
 #pod     $result = $self->fill_in_file( $file, \%variables, \%extra_args );
 #pod     $result = $self->fill_in_file( $file );
 #pod
-#pod Similar to C<fill_in_string>, but the first argument is not a string but a file. File content is
-#pod treated as template, and will be replaced with result of template processing. New file content will
-#pod also be returned.
+#pod Similar to C<fill_in_string>, but the first argument is not a template but a file object or file
+#pod name to read template from. File can be any of C<Dist::Zilla> file types (file read with C<content>
+#pod method) or C<Path::Tiny> file (file read with C<slurp_utf8> method), or C<Path::Class::File> (read
+#pod by C<< slurp( iomode => '<:encoding(UTF-8)' ) >>) or just a file name (C<Dist::Zilla::File::OnDisk>
+#pod object fill be created internally).
+#pod
+#pod The method returns result of template processing. If the file is mutable (i. e. does
+#pod C<Dist::Zilla::Role::MutableFile>) file content is also updated.
 #pod
 #pod =cut
 
 sub fill_in_file {
     my ( $self, $file, $hash, $args ) = @_;
-    my %args = (
-        filename => $file->name,
-        $args ? %$args : (),
-    );
-    return $file->content( $self->fill_in_string( $file->content, $hash, \%args ) );
+    my $class = blessed( $file );
+    if ( $class ) {
+        if ( $file->isa( 'Moose::Object' ) ) {
+            if ( $file->does( 'Dist::Zilla::Role::File' ) ) {
+                my %args = (
+                    filename => $file->name,
+                    $args ? %$args : (),
+                );
+                my $result = $self->fill_in_string( $file->content, $hash, \%args );
+                if ( $file->does( 'Dist::Zilla::Role::MutableFile' ) ) {
+                    return $file->content( $result );
+                } else {
+                    return $result;
+                };
+            };
+        } elsif ( $file->isa( 'Path::Tiny' ) ) {
+            my %args = (
+                filename => "$file",
+                $args ? %$args : (),
+            );
+            return $self->fill_in_string( $file->slurp_utf8(), $hash, \%args );
+        } elsif ( $file->isa( 'Path::Class::File' ) ) {
+            my %args = (
+                filename => "$file",
+                $args ? %$args : (),
+            );
+            my @mode = ( iomode => '<:encoding(UTF-8)' );
+            return $self->fill_in_string( $file->slurp( @mode ), $hash, \%args );
+        };
+    } else {
+        return $self->fill_in_file(
+            Dist::Zilla::File::OnDisk->new( {
+                name => $file
+            } ),
+            $hash,
+            $args
+        );
+    };
+    croak "fill_in_string: unsupported file class: $class";
+};
+
+# --------------------------------------------------------------------------------------------------
+
+#pod =method _show_context
+#pod
+#pod C<TextTemplater> uses this method to report errors in templates.
+#pod
+#pod     $self->_show_context( $text,
+#pod         $linenum0 => $message0,
+#pod         $linenum1 => $message1,
+#pod         ...
+#pod     );
+#pod
+#pod C<$text> is a text to show. It could be either a C<Str> (multiline) or C<ArrayRef[Str]> (a line per
+#pod element, either chomped or not). The method does not show entire text, but only problem ("focus")
+#pod lines in context (C<$context> lines above and below each focus line).
+#pod
+#pod Focus lines are specified by pairs C<< $linenum => $message >>, where C<$linenum> is a number of
+#pod problem line, and C<$message> is an error message to print just below the specified line. Multiple
+#pod focus lines are allowed. The same line number may be used several times, all the messages will be
+#pod printed in order of appearance. Line numbers less than 1 and corresponding messages are silently
+#pod ignored.
+#pod
+#pod =cut
+
+sub _show_context {
+    my ( $self, $text, @notes ) = @_;
+    #   TODO: Chop too long lines?
+    if ( not ref( $text ) ) {
+        $text  = [ split( "\n", $text ) ];
+    };
+    my %notes;
+    while ( @notes ) {
+        my ( $n, $msg ) = splice( @notes, 0, 2 );
+        if ( $n > 0 ) {
+            my $ctx = $self->tt_context;
+            for my $i ( max( $n - $ctx, 1 ) .. min( $n + $ctx, @$text + 0 ) ) {
+                if ( not $notes{ $i } ) {
+                    $notes{ $i } = [];
+                };
+            };
+            push( @{ $notes{ $n } }, $msg );
+        };
+    };
+    my $w        = length( 0 + @$text );        # Width of linenumber column.
+    my $indent = ' ' x 4;
+    my $fline  = $indent . '%0' . $w . 'd: %s';
+    my $fnote  = $indent . ( ' ' x $w ) . '  ^^^ %s ^^^';               # Notice line format.
+    my $fskip  = $indent . ( ' ' x $w ) . '  ...skipped %d lines...';   # "Skipped" notice format.
+    my $last  = 0;                          # Number of the last printed line.
+    my $show_line = sub {
+        my ( $n ) = @_;
+        my $line = $text->[ $n - 1 ];
+        chomp( $line );
+        $self->log_error( [ $fline, $n, $line ] );
+    };
+    my $show_note = sub {
+        my ( $n ) = @_;
+        $self->log_error( [ $fnote, $_ ] ) for @{ $notes{ $n } };
+    };
+    my $show_skip = sub {
+        my ( $n ) = @_;
+        if ( $n > $last + 1 ) {                 # There are skipped line.
+            my $count = $n - $last - 1;         # Number of skipped lines.
+            if ( $count == 1 ) {
+                $show_line->( $n - 1 );         # There is no sense to skip one line.
+            } else {
+                $self->log_error( [ $fskip, $count ] );
+            };
+        };
+    };
+    for my $n ( sort( { $a <=> $b } keys( %notes ) ) ) {
+        $show_skip->( $n );
+        $show_line->( $n );
+        $show_note->( $n );
+        $last = $n;
+    };
+    $show_skip->( @$text + 1 );
 };
 
 # --------------------------------------------------------------------------------------------------
@@ -506,6 +635,8 @@ sub fill_in_file {
 # end of file #
 # doc/why.pod #
 
+
+
 #pod =encoding UTF-8
 #pod
 #pod =head1 WHY?
@@ -515,53 +646,117 @@ sub fill_in_file {
 #pod
 #pod =head2 Error Reporting
 #pod
-#pod Let us consider an example. Have a look at F<dist.ini>:
+#pod Let us consider an example. For sake of example simplicity, it contains only one file, F<dist.ini>.
+#pod Two files, F<lib/Assa.pm> and F<lib/Assa.pod>, are generated on-the-fly with C<GenerateFile>
+#pod plugin.
+#pod
+#pod Have a look at F<dist.ini>:
 #pod
 #pod     name     = Assa
 #pod     version  = 0.001
-#pod     [GatherDir]
-#pod     [PruneCruft]
-#pod     [GenerateFile/COPYING]
-#pod         filename = COPYING
-#pod         content  = This is {{$dist->name}}.
+#pod     abstract = Example
+#pod     [GenerateFile/lib/Assa.pm]
+#pod         filename = lib/Assa.pm
+#pod         content  = package Assa; 1;
+#pod     [GenerateFile/lib/Assa/Manual.pod]
+#pod         filename = lib/Assa/Manual.pod
+#pod         content  = =head1 NAME
 #pod         content  =
-#pod         content  = {{$dst->license->notice}}
-#pod         is_template = 1
+#pod         content  = {{$dst->name} - {{$dist->abstract}}
+#pod         content  =
+#pod         content  = Version {{$dist->version}}.
+#pod         content  =
+#pod         content  = {{$dist->license->notice}}
 #pod     [TemplateFiles]
 #pod         filename = lib/Assa.pm
 #pod         filename = lib/Assa/Manual.pod
 #pod     [MetaResources::Template]
-#pod         license = {{$dist->license->url}}
-#pod         …
-#pod     …
+#pod         homepage = https://example.org/release/{{$dist->name}}
+#pod         license  = {{$dist->license->url}}
 #pod
-#pod (Do you see a typo? Real F<dist.ini> can be much longer.) Now let us build the distribution:
+#pod
+#pod (Do you see a typo? How many? Note this is a small example, real files are much larger.) Now let us
+#pod build the distribution:
 #pod
 #pod     $ dzil build
 #pod     [DZ] beginning to build Assa
-#pod     Can't call method "license" on an undefined value at template line 3.
+#pod     [TemplateFiles] Filling in the template returned undef for:
+#pod     [TemplateFiles] =head1 NAME
+#pod     [TemplateFiles]
+#pod     [TemplateFiles] {{$dst->name} - {{$dist->abstract}}
+#pod     [TemplateFiles]
+#pod     [TemplateFiles] Version {{$dist->version}}.
+#pod     [TemplateFiles]
+#pod     [TemplateFiles] {{$dist->license->notice}}
+#pod     [TemplateFiles] Filling in the template returned undef for:
+#pod     [TemplateFiles] =head1 NAME
+#pod     [TemplateFiles]
+#pod     [TemplateFiles] {{$dst->name} - {{$dist->abstract}}
+#pod     [TemplateFiles]
+#pod     [TemplateFiles] Version {{$dist->version}}.
+#pod     [TemplateFiles]
+#pod     [TemplateFiles] {{$dist->license->notice}}
+#pod      at /home/vdb/.usr/opt/local-lib/lib/perl5/x86_64-linux-thread-multi/Moose/Meta/Method/Delegation.pm line 110.
 #pod
-#pod Oops. Ok, it is clear what was happened, but where?? The only weak clue is word "template".
-#pod However, there are many templates in the project: F<lib/Assa.pm>, F<lib/Assa/Manual.pod>, generated
-#pod on-the-fly F<COPYING>, and even resources in F<META.yml> — all this stuff is generated from
-#pod templates. Where is the problem? Have a happy bug hunting, dude.
 #pod
-#pod If these plugins (namely, C<GenerateFile>, C<TemplateFiles> and C<MetaResources::Template>) were
-#pod written with C<TextTemplateB<r>> role, result would be:
+#pod Oops. What's happened? Where? Why? All we have is a highly unclear error message
+#pod
+#pod     Filling in the template returned undef for:
+#pod
+#pod and file content printed twice. (Yep, if the problem file had 1000 lines, we would have it printed
+#pod twice too.) We do not ever have a file name and have to guess it by the content. Good bug hunting,
+#pod dude.
+#pod
+#pod Ok, let us fix the problem (mistyped closing delimiter in the first line of file
+#pod F<lib/Assa/Manual.pod>) and build the distribution again:
 #pod
 #pod     $ dzil build
 #pod     [DZ] beginning to build Assa
-#pod     [COPYING] Can't call method "license" on an undefined value at COPYING line 3.
-#pod     [COPYING] Problem code fragment begins at COPYING line 3.
-#pod     [COPYING] COPYING:
-#pod     [COPYING]     1: This is {{$dist->name}}.
-#pod     [COPYING]     2:
-#pod     [COPYING]     3: {{$dst->license->notice}}
-#pod     [COPYING]        ^^^ Code fragment begins in line above ^^^
-#pod     [COPYING]        ^^^ Code fragment died in line above ^^^
+#pod     Can't call method "name" on an undefined value at template line 3.
+#pod
+#pod
+#pod Oops. Error message much is better now, but where the problem is? There are many templates in the
+#pod project: F<lib/Assa.pm>, F<lib/Assa/Manual.pod>, and even resources in F<META.yml> — all are
+#pod generated from templates. Where is the problem? Good bug hunting for us all.
+#pod
+#pod Such error reporting is simply unacceptable. I am a human, I often make mistakes, and I want the
+#pod tool clearly warns me I<what> and I<where> the problem is, so I can fix it quickly. For example,
+#pod in the first case I want to see:
+#pod
+#pod     $ dzil build
+#pod     [DZ] beginning to build Assa
+#pod     [Templates] Unmatched opening delimiter at lib/Assa/Manual.pod line 3.
+#pod     [Templates] lib/Assa/Manual.pod:
+#pod     [Templates]     1: =head1 NAME
+#pod     [Templates]     2:
+#pod     [Templates]     3: {{$dst->name} - {{$dist->abstract}}
+#pod     [Templates]        ^^^ Unmatched opening delimiter in line above ^^^
+#pod     [Templates]     4:
+#pod     [Templates]     5: Version {{$dist->version}}.
+#pod     [Templates]        ...skipped 2 lines...
 #pod     Aborting...
 #pod
-#pod See the difference.
+#pod
+#pod In the second case:
+#pod
+#pod     $ dzil build
+#pod     [DZ] beginning to build Assa
+#pod     [Templates] Can't call method "name" on an undefined value at lib/Assa/Manual.pod line 3.
+#pod     [Templates] Problem code fragment begins at lib/Assa/Manual.pod line 3.
+#pod     [Templates] lib/Assa/Manual.pod:
+#pod     [Templates]     1: =head1 NAME
+#pod     [Templates]     2:
+#pod     [Templates]     3: {{$dst->name}} - {{$dist->abstract}}
+#pod     [Templates]        ^^^ Code fragment begins in line above ^^^
+#pod     [Templates]        ^^^ Code fragment died in line above ^^^
+#pod     [Templates]     4:
+#pod     [Templates]     5: Version {{$dist->version}}.
+#pod     [Templates]        ...skipped 2 lines...
+#pod     Aborting...
+#pod
+#pod
+#pod C<TextTemplater> makes it real. All I need is using C<TextTemplater>-based plugins: C<Templates>,
+#pod C<MetaResources::Template> (starting from v0.002).
 #pod
 #pod =head2 Engine Control
 #pod
@@ -588,7 +783,7 @@ Dist::Zilla::Role::TextTemplater - Have text templating capabilities in your Dis
 
 =head1 VERSION
 
-Version 0.005, released on 2015-08-12 08:18 UTC.
+Version 0.006, released on 2015-08-13 18:44 UTC.
 
 =head1 WHAT?
 
@@ -601,7 +796,7 @@ have text templating capabilities in your Dist::Zilla plugin.
 
 If you are using a C<TextTemplater>-based plugin, read the
 L<manual|Dist::Zilla::Role::TextTemplater::Manual>. General topics like getting source, building, installing, bug
-reporting and some others are covered in the L<readme|Dist::Zilla::Role::TextTemplater::ReadMe>.
+reporting and some others are covered in the F<README>.
 
 =head1 SYNOPSIS
 
@@ -695,6 +890,14 @@ C<broken> subroutine).
 
 C<Str>, read-only, not an init arg.
 
+=head2 tt_context
+
+Number of template lines to show above and below error line in error messages.
+
+C<Int>, read-only, not an init arg, default value 2.
+
+There is no (official) way to change the attribute value now. Let me know if you need it.
+
 =head1 OBJECT METHODS
 
 =head2 mvp_multivalue_args
@@ -735,13 +938,44 @@ It also allows to cancel defining C<$plugin> and C<$dist> variables, e. g.:
 
 =head2 fill_in_file
 
-    $file = Dist::Zilla::File::OnDisk->new( { … } );
+    $file = Dist::Zilla::File::OnDisk->new( { … } );    # or
+    $file = Dist::Zilla::File::InMemory->new( { … } );  # or
+    $file = Dist::Zilla::File::FromCode->new( { … } );  # or
+    $file = Path::Tiny->new( 'filename' );              # or
+    $file = Path::Class::File->new( 'filename' );       # or
+    $file = 'filename.ext';
+
     $result = $self->fill_in_file( $file, \%variables, \%extra_args );
     $result = $self->fill_in_file( $file );
 
-Similar to C<fill_in_string>, but the first argument is not a string but a file. File content is
-treated as template, and will be replaced with result of template processing. New file content will
-also be returned.
+Similar to C<fill_in_string>, but the first argument is not a template but a file object or file
+name to read template from. File can be any of C<Dist::Zilla> file types (file read with C<content>
+method) or C<Path::Tiny> file (file read with C<slurp_utf8> method), or C<Path::Class::File> (read
+by C<< slurp( iomode => '<:encoding(UTF-8)' ) >>) or just a file name (C<Dist::Zilla::File::OnDisk>
+object fill be created internally).
+
+The method returns result of template processing. If the file is mutable (i. e. does
+C<Dist::Zilla::Role::MutableFile>) file content is also updated.
+
+=head2 _show_context
+
+C<TextTemplater> uses this method to report errors in templates.
+
+    $self->_show_context( $text,
+        $linenum0 => $message0,
+        $linenum1 => $message1,
+        ...
+    );
+
+C<$text> is a text to show. It could be either a C<Str> (multiline) or C<ArrayRef[Str]> (a line per
+element, either chomped or not). The method does not show entire text, but only problem ("focus")
+lines in context (C<$context> lines above and below each focus line).
+
+Focus lines are specified by pairs C<< $linenum => $message >>, where C<$linenum> is a number of
+problem line, and C<$message> is an error message to print just below the specified line. Multiple
+focus lines are allowed. The same line number may be used several times, all the messages will be
+printed in order of appearance. Line numbers less than 1 and corresponding messages are silently
+ignored.
 
 =head1 NOTES
 
@@ -821,53 +1055,112 @@ engine under the hood, but lacks of control and has I<awful> error reporting.
 
 =head2 Error Reporting
 
-Let us consider an example. Have a look at F<dist.ini>:
+Let us consider an example. For sake of example simplicity, it contains only one file, F<dist.ini>.
+Two files, F<lib/Assa.pm> and F<lib/Assa.pod>, are generated on-the-fly with C<GenerateFile>
+plugin.
+
+Have a look at F<dist.ini>:
 
     name     = Assa
     version  = 0.001
-    [GatherDir]
-    [PruneCruft]
-    [GenerateFile/COPYING]
-        filename = COPYING
-        content  = This is {{$dist->name}}.
+    abstract = Example
+    [GenerateFile/lib/Assa.pm]
+        filename = lib/Assa.pm
+        content  = package Assa; 1;
+    [GenerateFile/lib/Assa/Manual.pod]
+        filename = lib/Assa/Manual.pod
+        content  = =head1 NAME
         content  =
-        content  = {{$dst->license->notice}}
-        is_template = 1
+        content  = {{$dst->name} - {{$dist->abstract}}
+        content  =
+        content  = Version {{$dist->version}}.
+        content  =
+        content  = {{$dist->license->notice}}
     [TemplateFiles]
         filename = lib/Assa.pm
         filename = lib/Assa/Manual.pod
     [MetaResources::Template]
-        license = {{$dist->license->url}}
-        …
-    …
+        homepage = https://example.org/release/{{$dist->name}}
+        license  = {{$dist->license->url}}
 
-(Do you see a typo? Real F<dist.ini> can be much longer.) Now let us build the distribution:
+(Do you see a typo? How many? Note this is a small example, real files are much larger.) Now let us
+build the distribution:
 
     $ dzil build
     [DZ] beginning to build Assa
-    Can't call method "license" on an undefined value at template line 3.
+    [TemplateFiles] Filling in the template returned undef for:
+    [TemplateFiles] =head1 NAME
+    [TemplateFiles]
+    [TemplateFiles] {{$dst->name} - {{$dist->abstract}}
+    [TemplateFiles]
+    [TemplateFiles] Version {{$dist->version}}.
+    [TemplateFiles]
+    [TemplateFiles] {{$dist->license->notice}}
+    [TemplateFiles] Filling in the template returned undef for:
+    [TemplateFiles] =head1 NAME
+    [TemplateFiles]
+    [TemplateFiles] {{$dst->name} - {{$dist->abstract}}
+    [TemplateFiles]
+    [TemplateFiles] Version {{$dist->version}}.
+    [TemplateFiles]
+    [TemplateFiles] {{$dist->license->notice}}
+     at /home/vdb/.usr/opt/local-lib/lib/perl5/x86_64-linux-thread-multi/Moose/Meta/Method/Delegation.pm line 110.
 
-Oops. Ok, it is clear what was happened, but where?? The only weak clue is word "template".
-However, there are many templates in the project: F<lib/Assa.pm>, F<lib/Assa/Manual.pod>, generated
-on-the-fly F<COPYING>, and even resources in F<META.yml> — all this stuff is generated from
-templates. Where is the problem? Have a happy bug hunting, dude.
+Oops. What's happened? Where? Why? All we have is a highly unclear error message
 
-If these plugins (namely, C<GenerateFile>, C<TemplateFiles> and C<MetaResources::Template>) were
-written with C<TextTemplateB<r>> role, result would be:
+    Filling in the template returned undef for:
+
+and file content printed twice. (Yep, if the problem file had 1000 lines, we would have it printed
+twice too.) We do not ever have a file name and have to guess it by the content. Good bug hunting,
+dude.
+
+Ok, let us fix the problem (mistyped closing delimiter in the first line of file
+F<lib/Assa/Manual.pod>) and build the distribution again:
 
     $ dzil build
     [DZ] beginning to build Assa
-    [COPYING] Can't call method "license" on an undefined value at COPYING line 3.
-    [COPYING] Problem code fragment begins at COPYING line 3.
-    [COPYING] COPYING:
-    [COPYING]     1: This is {{$dist->name}}.
-    [COPYING]     2:
-    [COPYING]     3: {{$dst->license->notice}}
-    [COPYING]        ^^^ Code fragment begins in line above ^^^
-    [COPYING]        ^^^ Code fragment died in line above ^^^
+    Can't call method "name" on an undefined value at template line 3.
+
+Oops. Error message much is better now, but where the problem is? There are many templates in the
+project: F<lib/Assa.pm>, F<lib/Assa/Manual.pod>, and even resources in F<META.yml> — all are
+generated from templates. Where is the problem? Good bug hunting for us all.
+
+Such error reporting is simply unacceptable. I am a human, I often make mistakes, and I want the
+tool clearly warns me I<what> and I<where> the problem is, so I can fix it quickly. For example,
+in the first case I want to see:
+
+    $ dzil build
+    [DZ] beginning to build Assa
+    [Templates] Unmatched opening delimiter at lib/Assa/Manual.pod line 3.
+    [Templates] lib/Assa/Manual.pod:
+    [Templates]     1: =head1 NAME
+    [Templates]     2:
+    [Templates]     3: {{$dst->name} - {{$dist->abstract}}
+    [Templates]        ^^^ Unmatched opening delimiter in line above ^^^
+    [Templates]     4:
+    [Templates]     5: Version {{$dist->version}}.
+    [Templates]        ...skipped 2 lines...
     Aborting...
 
-See the difference.
+In the second case:
+
+    $ dzil build
+    [DZ] beginning to build Assa
+    [Templates] Can't call method "name" on an undefined value at lib/Assa/Manual.pod line 3.
+    [Templates] Problem code fragment begins at lib/Assa/Manual.pod line 3.
+    [Templates] lib/Assa/Manual.pod:
+    [Templates]     1: =head1 NAME
+    [Templates]     2:
+    [Templates]     3: {{$dst->name}} - {{$dist->abstract}}
+    [Templates]        ^^^ Code fragment begins in line above ^^^
+    [Templates]        ^^^ Code fragment died in line above ^^^
+    [Templates]     4:
+    [Templates]     5: Version {{$dist->version}}.
+    [Templates]        ...skipped 2 lines...
+    Aborting...
+
+C<TextTemplater> makes it real. All I need is using C<TextTemplater>-based plugins: C<Templates>,
+C<MetaResources::Template> (starting from v0.002).
 
 =head2 Engine Control
 
