@@ -27,6 +27,9 @@ package PluginTester;
 #   testing plugin for `Dist::Zilla`, and `Dist-Zilla` is also `Moose`-based.
 
 use autodie ':all';
+use Moose 2.0800 ();
+    # ^ Test will likely fail with older Moose.
+    #   Starting from 2.0800 "Roles can now override methods from other roles…".
 use namespace::autoclean;
 
 use Test::Routine;
